import os, stat, errno, subprocess, socket, platform
import logging
import time
from px_server.helper import download_and_extract_px_server_package

logging.getLogger().setLevel(logging.INFO)


class Server:

    __instance = None

    __instances = {}

    @staticmethod
    def get_instance(port=None):
        instance = None
        if port is None:
            if Server.__instance is None:
                Server.__instance = Server()
            instance = Server.__instance
        else:
            instance = Server.__instances[port]
            if instance is None:
                instance = Server(port)
                Server.__instances[port] = instance
        return instance

    port = None

    process = None

    px_server_executable_file_path = None

    def __init__(self, port=None):
        self.port = port
        self.setup()
        self.start()

    def __del__(self):
        self.stop()

    def setup(self):
        self.px_server_executable_file_path = download_and_extract_px_server_package()

    def start(self):

        if self.port is None:
            s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            s.bind(("", 0))
            self.port = s.getsockname()[1]

        logging.info('Start px server from {} file on port {}'.format(self.px_server_executable_file_path, self.port))
        self.process = subprocess.Popen([self.px_server_executable_file_path, '--port', str(self.port)])
        time.sleep(1)

    def stop(self):
        if self.process is not None:
            logging.info('Stop px server')
            logging.info('Stop px server process {}'.format(self.process.pid))
            self.process.terminate()
            logging.info('px server stopped')



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: px/px.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='px/px.proto',
  package='px',
  syntax='proto3',
  serialized_options=b'\n\tc.k.px.pxB\014BrowserProtoP\001',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x0bpx/px.proto\x12\x02px\"\xde\n\n\x06\x41\x63tion\x12\x34\n\x12\x63learAndTypeAction\x18\x0c \x01(\x0b\x32\x16.px.ClearAndTypeActionH\x00\x12&\n\x0b\x63lickAction\x18\x03 \x01(\x0b\x32\x0f.px.ClickActionH\x00\x12N\n\x1f\x63lickAndWaitForNavigationAction\x18\x1b \x01(\x0b\x32#.px.ClickAndWaitForNavigationActionH\x00\x12&\n\x0b\x63loseAction\x18\x07 \x01(\x0b\x32\x0f.px.CloseActionH\x00\x12*\n\rcookiesAction\x18\x18 \x01(\x0b\x32\x11.px.CookiesActionH\x00\x12\x34\n\x12\x64\x65leteCookieAction\x18\x1a \x01(\x0b\x32\x16.px.DeleteCookieActionH\x00\x12*\n\remulateAction\x18\x14 \x01(\x0b\x32\x11.px.EmulateActionH\x00\x12<\n\x16\x65mulateMediaTypeAction\x18\x17 \x01(\x0b\x32\x1a.px.EmulateMediaTypeActionH\x00\x12,\n\x0e\x65valuateAction\x18\x11 \x01(\x0b\x32\x12.px.EvaluateActionH\x00\x12&\n\x0b\x66ocusAction\x18\x0b \x01(\x0b\x32\x0f.px.FocusActionH\x00\x12\x34\n\x12getInnerTextAction\x18\x06 \x01(\x0b\x32\x16.px.GetInnerTextActionH\x00\x12$\n\ngotoAction\x18\x02 \x01(\x0b\x32\x0e.px.GotoActionH\x00\x12*\n\rkeyDownAction\x18\t \x01(\x0b\x32\x11.px.KeyDownActionH\x00\x12,\n\x0ekeyPressAction\x18\x08 \x01(\x0b\x32\x12.px.KeyPressActionH\x00\x12&\n\x0bkeyUpAction\x18\n \x01(\x0b\x32\x0f.px.KeyUpActionH\x00\x12(\n\x0claunchAction\x18\x01 \x01(\x0b\x32\x10.px.LaunchActionH\x00\x12\"\n\tpdfAction\x18\x16 \x01(\x0b\x32\r.px.PdfActionH\x00\x12(\n\x0creloadAction\x18\x12 \x01(\x0b\x32\x10.px.ReloadActionH\x00\x12(\n\x0cselectAction\x18\x05 \x01(\x0b\x32\x10.px.SelectActionH\x00\x12.\n\x0fsetCookieAction\x18\x19 \x01(\x0b\x32\x13.px.SetCookieActionH\x00\x12\x32\n\x11setViewportAction\x18\x13 \x01(\x0b\x32\x15.px.SetViewportActionH\x00\x12\x34\n\x12startTracingAction\x18\x0e \x01(\x0b\x32\x16.px.StartTracingActionH\x00\x12\x32\n\x11stopTracingAction\x18\x0f \x01(\x0b\x32\x15.px.StopTracingActionH\x00\x12\x30\n\x10screenshotAction\x18\r \x01(\x0b\x32\x14.px.ScreenshotActionH\x00\x12\"\n\ttapAction\x18\x15 \x01(\x0b\x32\r.px.TapActionH\x00\x12$\n\ntypeAction\x18\x04 \x01(\x0b\x32\x0e.px.TypeActionH\x00\x12$\n\nwaitAction\x18\x10 \x01(\x0b\x32\x0e.px.WaitActionH\x00\x12*\n\rwaitForAction\x18\x1c \x01(\x0b\x32\x11.px.WaitForActionH\x00\x12\x30\n\x10uploadFileAction\x18\x1d \x01(\x0b\x32\x14.px.UploadFileActionH\x00\x42\x0e\n\x0c\x61\x63tion_oneof\"4\n\x12\x43learAndTypeAction\x12\x10\n\x08selector\x18\x01 \x01(\t\x12\x0c\n\x04text\x18\x02 \x01(\t\"@\n\x0b\x43lickAction\x12\x10\n\x08selector\x18\x01 \x01(\t\x12\x1f\n\x06option\x18\x02 \x01(\x0b\x32\x0f.px.ClickOption\"x\n\x1f\x43lickAndWaitForNavigationAction\x12\x10\n\x08selector\x18\x01 \x01(\t\x12\x1f\n\x06option\x18\x02 \x01(\x0b\x32\x0f.px.ClickOption\x12\"\n\nwaitOption\x18\x03 \x01(\x0b\x32\x0e.px.WaitOption\"\r\n\x0b\x43loseAction\"\x0f\n\rCookiesAction\"\"\n\x12\x44\x65leteCookieAction\x12\x0c\n\x04name\x18\x01 \x01(\t\"(\n\tDoRequest\x12\x1b\n\x07\x61\x63tions\x18\x01 \x03(\x0b\x32\n.px.Action\"t\n\nDoResponse\x12\x0e\n\x06result\x18\x01 \x01(\t\x12\r\n\x05\x65rror\x18\x02 \x01(\t\x12%\n\x06status\x18\x03 \x01(\x0e\x32\x15.px.DoResponse.Status\" \n\x06Status\x12\x0b\n\x07SUCCESS\x10\x00\x12\t\n\x05\x45RROR\x10\x01\"\x1f\n\rEmulateAction\x12\x0e\n\x06\x64\x65vice\x18\x01 \x01(\t\"&\n\x16\x45mulateMediaTypeAction\x12\x0c\n\x04type\x18\x01 \x01(\t\"&\n\x0e\x45valuateAction\x12\x14\n\x0cpageFunction\x18\x01 \x01(\t\"\x1f\n\x0b\x46ocusAction\x12\x10\n\x08selector\x18\x01 \x01(\t\"&\n\x12GetInnerTextAction\x12\x10\n\x08selector\x18\x01 \x01(\t\"\x19\n\nGotoAction\x12\x0b\n\x03url\x18\x01 \x01(\t\"\x1c\n\rKeyDownAction\x12\x0b\n\x03key\x18\x01 \x01(\t\"\x1d\n\x0eKeyPressAction\x12\x0b\n\x03key\x18\x01 \x01(\t\"\x1a\n\x0bKeyUpAction\x12\x0b\n\x03key\x18\x01 \x01(\t\"U\n\x0cLaunchAction\x12\x10\n\x08headless\x18\x01 \x01(\x08\x12%\n\x0f\x64\x65\x66\x61ultViewport\x18\x02 \x01(\x0b\x32\x0c.px.Viewport\x12\x0c\n\x04\x61rgs\x18\x03 \x03(\t\")\n\tPdfAction\x12\x0c\n\x04path\x18\x01 \x01(\t\x12\x0e\n\x06\x66ormat\x18\x02 \x01(\t\"\x0e\n\x0cReloadAction\"0\n\x0cSelectAction\x12\x10\n\x08selector\x18\x01 \x01(\t\x12\x0e\n\x06values\x18\x02 \x03(\t\".\n\x0fSetCookieAction\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"2\n\x11SetViewportAction\x12\r\n\x05width\x18\x01 \x01(\x05\x12\x0e\n\x06height\x18\x02 \x01(\x05\"7\n\x12StartTracingAction\x12\x0c\n\x04path\x18\x01 \x01(\t\x12\x13\n\x0bscreenshots\x18\x02 \x01(\x08\"\x13\n\x11StopTracingAction\"2\n\x10ScreenshotAction\x12\x0c\n\x04path\x18\x01 \x01(\t\x12\x10\n\x08\x66ullPage\x18\x02 \x01(\x08\"\x1d\n\tTapAction\x12\x10\n\x08selector\x18\x01 \x01(\t\",\n\nTypeAction\x12\x10\n\x08selector\x18\x01 \x01(\t\x12\x0c\n\x04text\x18\x02 \x01(\t\"\x1a\n\nWaitAction\x12\x0c\n\x04time\x18\x01 \x01(\x05\"!\n\rWaitForAction\x12\x10\n\x08selector\x18\x01 \x01(\t\"3\n\x10UploadFileAction\x12\x10\n\x08selector\x18\x01 \x01(\t\x12\r\n\x05paths\x18\x02 \x03(\t\"1\n\x0b\x43lickOption\x12\x0e\n\x06\x62utton\x18\x01 \x01(\t\x12\x12\n\nclickCount\x18\x02 \x01(\x05\"0\n\nWaitOption\x12\x0f\n\x07timeout\x18\x01 \x01(\x05\x12\x11\n\twaitUntil\x18\x02 \x01(\t\"}\n\x08Viewport\x12\r\n\x05width\x18\x01 \x01(\x05\x12\x0e\n\x06height\x18\x02 \x01(\x05\x12\x19\n\x11\x64\x65viceScaleFactor\x18\x03 \x01(\x05\x12\x10\n\x08isMobile\x18\x04 \x01(\x08\x12\x10\n\x08hasTouch\x18\x05 \x01(\x08\x12\x13\n\x0bisLandscape\x18\x06 \x01(\x08\x32\x30\n\x07\x42rowser\x12%\n\x02\x44o\x12\r.px.DoRequest\x1a\x0e.px.DoResponse\"\x00\x42\x1b\n\tc.k.px.pxB\x0c\x42rowserProtoP\x01\x62\x06proto3'
)



_DORESPONSE_STATUS = _descriptor.EnumDescriptor(
  name='Status',
  full_name='px.DoResponse.Status',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='SUCCESS', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ERROR', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1832,
  serialized_end=1864,
)
_sym_db.RegisterEnumDescriptor(_DORESPONSE_STATUS)


_ACTION = _descriptor.Descriptor(
  name='Action',
  full_name='px.Action',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='clearAndTypeAction', full_name='px.Action.clearAndTypeAction', index=0,
      number=12, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='clickAction', full_name='px.Action.clickAction', index=1,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='clickAndWaitForNavigationAction', full_name='px.Action.clickAndWaitForNavigationAction', index=2,
      number=27, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='closeAction', full_name='px.Action.closeAction', index=3,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='cookiesAction', full_name='px.Action.cookiesAction', index=4,
      number=24, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='deleteCookieAction', full_name='px.Action.deleteCookieAction', index=5,
      number=26, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='emulateAction', full_name='px.Action.emulateAction', index=6,
      number=20, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='emulateMediaTypeAction', full_name='px.Action.emulateMediaTypeAction', index=7,
      number=23, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='evaluateAction', full_name='px.Action.evaluateAction', index=8,
      number=17, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='focusAction', full_name='px.Action.focusAction', index=9,
      number=11, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='getInnerTextAction', full_name='px.Action.getInnerTextAction', index=10,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='gotoAction', full_name='px.Action.gotoAction', index=11,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='keyDownAction', full_name='px.Action.keyDownAction', index=12,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='keyPressAction', full_name='px.Action.keyPressAction', index=13,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='keyUpAction', full_name='px.Action.keyUpAction', index=14,
      number=10, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='launchAction', full_name='px.Action.launchAction', index=15,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='pdfAction', full_name='px.Action.pdfAction', index=16,
      number=22, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='reloadAction', full_name='px.Action.reloadAction', index=17,
      number=18, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='selectAction', full_name='px.Action.selectAction', index=18,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='setCookieAction', full_name='px.Action.setCookieAction', index=19,
      number=25, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='setViewportAction', full_name='px.Action.setViewportAction', index=20,
      number=19, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='startTracingAction', full_name='px.Action.startTracingAction', index=21,
      number=14, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='stopTracingAction', full_name='px.Action.stopTracingAction', index=22,
      number=15, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='screenshotAction', full_name='px.Action.screenshotAction', index=23,
      number=13, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='tapAction', full_name='px.Action.tapAction', index=24,
      number=21, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='typeAction', full_name='px.Action.typeAction', index=25,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='waitAction', full_name='px.Action.waitAction', index=26,
      number=16, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='waitForAction', full_name='px.Action.waitForAction', index=27,
      number=28, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='uploadFileAction', full_name='px.Action.uploadFileAction', index=28,
      number=29, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='action_oneof', full_name='px.Action.action_oneof',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=20,
  serialized_end=1394,
)


_CLEARANDTYPEACTION = _descriptor.Descriptor(
  name='ClearAndTypeAction',
  full_name='px.ClearAndTypeAction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='selector', full_name='px.ClearAndTypeAction.selector', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='text', full_name='px.ClearAndTypeAction.text', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1396,
  serialized_end=1448,
)


_CLICKACTION = _descriptor.Descriptor(
  name='ClickAction',
  full_name='px.ClickAction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='selector', full_name='px.ClickAction.selector', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='option', full_name='px.ClickAction.option', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1450,
  serialized_end=1514,
)


_CLICKANDWAITFORNAVIGATIONACTION = _descriptor.Descriptor(
  name='ClickAndWaitForNavigationAction',
  full_name='px.ClickAndWaitForNavigationAction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='selector', full_name='px.ClickAndWaitForNavigationAction.selector', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='option', full_name='px.ClickAndWaitForNavigationAction.option', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='waitOption', full_name='px.ClickAndWaitForNavigationAction.waitOption', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1516,
  serialized_end=1636,
)


_CLOSEACTION = _descriptor.Descriptor(
  name='CloseAction',
  full_name='px.CloseAction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1638,
  serialized_end=1651,
)


_COOKIESACTION = _descriptor.Descriptor(
  name='CookiesAction',
  full_name='px.CookiesAction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1653,
  serialized_end=1668,
)


_DELETECOOKIEACTION = _descriptor.Descriptor(
  name='DeleteCookieAction',
  full_name='px.DeleteCookieAction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='px.DeleteCookieAction.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1670,
  serialized_end=1704,
)


_DOREQUEST = _descriptor.Descriptor(
  name='DoRequest',
  full_name='px.DoRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='actions', full_name='px.DoRequest.actions', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1706,
  serialized_end=1746,
)


_DORESPONSE = _descriptor.Descriptor(
  name='DoResponse',
  full_name='px.DoResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='result', full_name='px.DoResponse.result', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='error', full_name='px.DoResponse.error', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='status', full_name='px.DoResponse.status', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _DORESPONSE_STATUS,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1748,
  serialized_end=1864,
)


_EMULATEACTION = _descriptor.Descriptor(
  name='EmulateAction',
  full_name='px.EmulateAction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='device', full_name='px.EmulateAction.device', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1866,
  serialized_end=1897,
)


_EMULATEMEDIATYPEACTION = _descriptor.Descriptor(
  name='EmulateMediaTypeAction',
  full_name='px.EmulateMediaTypeAction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='px.EmulateMediaTypeAction.type', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1899,
  serialized_end=1937,
)


_EVALUATEACTION = _descriptor.Descriptor(
  name='EvaluateAction',
  full_name='px.EvaluateAction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='pageFunction', full_name='px.EvaluateAction.pageFunction', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1939,
  serialized_end=1977,
)


_FOCUSACTION = _descriptor.Descriptor(
  name='FocusAction',
  full_name='px.FocusAction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='selector', full_name='px.FocusAction.selector', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1979,
  serialized_end=2010,
)


_GETINNERTEXTACTION = _descriptor.Descriptor(
  name='GetInnerTextAction',
  full_name='px.GetInnerTextAction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='selector', full_name='px.GetInnerTextAction.selector', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2012,
  serialized_end=2050,
)


_GOTOACTION = _descriptor.Descriptor(
  name='GotoAction',
  full_name='px.GotoAction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='url', full_name='px.GotoAction.url', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2052,
  serialized_end=2077,
)


_KEYDOWNACTION = _descriptor.Descriptor(
  name='KeyDownAction',
  full_name='px.KeyDownAction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='px.KeyDownAction.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2079,
  serialized_end=2107,
)


_KEYPRESSACTION = _descriptor.Descriptor(
  name='KeyPressAction',
  full_name='px.KeyPressAction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='px.KeyPressAction.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2109,
  serialized_end=2138,
)


_KEYUPACTION = _descriptor.Descriptor(
  name='KeyUpAction',
  full_name='px.KeyUpAction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='px.KeyUpAction.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2140,
  serialized_end=2166,
)


_LAUNCHACTION = _descriptor.Descriptor(
  name='LaunchAction',
  full_name='px.LaunchAction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='headless', full_name='px.LaunchAction.headless', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='defaultViewport', full_name='px.LaunchAction.defaultViewport', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='args', full_name='px.LaunchAction.args', index=2,
      number=3, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2168,
  serialized_end=2253,
)


_PDFACTION = _descriptor.Descriptor(
  name='PdfAction',
  full_name='px.PdfAction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='path', full_name='px.PdfAction.path', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='format', full_name='px.PdfAction.format', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2255,
  serialized_end=2296,
)


_RELOADACTION = _descriptor.Descriptor(
  name='ReloadAction',
  full_name='px.ReloadAction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2298,
  serialized_end=2312,
)


_SELECTACTION = _descriptor.Descriptor(
  name='SelectAction',
  full_name='px.SelectAction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='selector', full_name='px.SelectAction.selector', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='values', full_name='px.SelectAction.values', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2314,
  serialized_end=2362,
)


_SETCOOKIEACTION = _descriptor.Descriptor(
  name='SetCookieAction',
  full_name='px.SetCookieAction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='px.SetCookieAction.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='px.SetCookieAction.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2364,
  serialized_end=2410,
)


_SETVIEWPORTACTION = _descriptor.Descriptor(
  name='SetViewportAction',
  full_name='px.SetViewportAction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='width', full_name='px.SetViewportAction.width', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='height', full_name='px.SetViewportAction.height', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2412,
  serialized_end=2462,
)


_STARTTRACINGACTION = _descriptor.Descriptor(
  name='StartTracingAction',
  full_name='px.StartTracingAction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='path', full_name='px.StartTracingAction.path', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='screenshots', full_name='px.StartTracingAction.screenshots', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2464,
  serialized_end=2519,
)


_STOPTRACINGACTION = _descriptor.Descriptor(
  name='StopTracingAction',
  full_name='px.StopTracingAction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2521,
  serialized_end=2540,
)


_SCREENSHOTACTION = _descriptor.Descriptor(
  name='ScreenshotAction',
  full_name='px.ScreenshotAction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='path', full_name='px.ScreenshotAction.path', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='fullPage', full_name='px.ScreenshotAction.fullPage', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2542,
  serialized_end=2592,
)


_TAPACTION = _descriptor.Descriptor(
  name='TapAction',
  full_name='px.TapAction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='selector', full_name='px.TapAction.selector', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2594,
  serialized_end=2623,
)


_TYPEACTION = _descriptor.Descriptor(
  name='TypeAction',
  full_name='px.TypeAction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='selector', full_name='px.TypeAction.selector', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='text', full_name='px.TypeAction.text', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2625,
  serialized_end=2669,
)


_WAITACTION = _descriptor.Descriptor(
  name='WaitAction',
  full_name='px.WaitAction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='time', full_name='px.WaitAction.time', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2671,
  serialized_end=2697,
)


_WAITFORACTION = _descriptor.Descriptor(
  name='WaitForAction',
  full_name='px.WaitForAction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='selector', full_name='px.WaitForAction.selector', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2699,
  serialized_end=2732,
)


_UPLOADFILEACTION = _descriptor.Descriptor(
  name='UploadFileAction',
  full_name='px.UploadFileAction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='selector', full_name='px.UploadFileAction.selector', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='paths', full_name='px.UploadFileAction.paths', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2734,
  serialized_end=2785,
)


_CLICKOPTION = _descriptor.Descriptor(
  name='ClickOption',
  full_name='px.ClickOption',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='button', full_name='px.ClickOption.button', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='clickCount', full_name='px.ClickOption.clickCount', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2787,
  serialized_end=2836,
)


_WAITOPTION = _descriptor.Descriptor(
  name='WaitOption',
  full_name='px.WaitOption',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='timeout', full_name='px.WaitOption.timeout', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='waitUntil', full_name='px.WaitOption.waitUntil', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2838,
  serialized_end=2886,
)


_VIEWPORT = _descriptor.Descriptor(
  name='Viewport',
  full_name='px.Viewport',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='width', full_name='px.Viewport.width', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='height', full_name='px.Viewport.height', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='deviceScaleFactor', full_name='px.Viewport.deviceScaleFactor', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='isMobile', full_name='px.Viewport.isMobile', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='hasTouch', full_name='px.Viewport.hasTouch', index=4,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='isLandscape', full_name='px.Viewport.isLandscape', index=5,
      number=6, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2888,
  serialized_end=3013,
)

_ACTION.fields_by_name['clearAndTypeAction'].message_type = _CLEARANDTYPEACTION
_ACTION.fields_by_name['clickAction'].message_type = _CLICKACTION
_ACTION.fields_by_name['clickAndWaitForNavigationAction'].message_type = _CLICKANDWAITFORNAVIGATIONACTION
_ACTION.fields_by_name['closeAction'].message_type = _CLOSEACTION
_ACTION.fields_by_name['cookiesAction'].message_type = _COOKIESACTION
_ACTION.fields_by_name['deleteCookieAction'].message_type = _DELETECOOKIEACTION
_ACTION.fields_by_name['emulateAction'].message_type = _EMULATEACTION
_ACTION.fields_by_name['emulateMediaTypeAction'].message_type = _EMULATEMEDIATYPEACTION
_ACTION.fields_by_name['evaluateAction'].message_type = _EVALUATEACTION
_ACTION.fields_by_name['focusAction'].message_type = _FOCUSACTION
_ACTION.fields_by_name['getInnerTextAction'].message_type = _GETINNERTEXTACTION
_ACTION.fields_by_name['gotoAction'].message_type = _GOTOACTION
_ACTION.fields_by_name['keyDownAction'].message_type = _KEYDOWNACTION
_ACTION.fields_by_name['keyPressAction'].message_type = _KEYPRESSACTION
_ACTION.fields_by_name['keyUpAction'].message_type = _KEYUPACTION
_ACTION.fields_by_name['launchAction'].message_type = _LAUNCHACTION
_ACTION.fields_by_name['pdfAction'].message_type = _PDFACTION
_ACTION.fields_by_name['reloadAction'].message_type = _RELOADACTION
_ACTION.fields_by_name['selectAction'].message_type = _SELECTACTION
_ACTION.fields_by_name['setCookieAction'].message_type = _SETCOOKIEACTION
_ACTION.fields_by_name['setViewportAction'].message_type = _SETVIEWPORTACTION
_ACTION.fields_by_name['startTracingAction'].message_type = _STARTTRACINGACTION
_ACTION.fields_by_name['stopTracingAction'].message_type = _STOPTRACINGACTION
_ACTION.fields_by_name['screenshotAction'].message_type = _SCREENSHOTACTION
_ACTION.fields_by_name['tapAction'].message_type = _TAPACTION
_ACTION.fields_by_name['typeAction'].message_type = _TYPEACTION
_ACTION.fields_by_name['waitAction'].message_type = _WAITACTION
_ACTION.fields_by_name['waitForAction'].message_type = _WAITFORACTION
_ACTION.fields_by_name['uploadFileAction'].message_type = _UPLOADFILEACTION
_ACTION.oneofs_by_name['action_oneof'].fields.append(
  _ACTION.fields_by_name['clearAndTypeAction'])
_ACTION.fields_by_name['clearAndTypeAction'].containing_oneof = _ACTION.oneofs_by_name['action_oneof']
_ACTION.oneofs_by_name['action_oneof'].fields.append(
  _ACTION.fields_by_name['clickAction'])
_ACTION.fields_by_name['clickAction'].containing_oneof = _ACTION.oneofs_by_name['action_oneof']
_ACTION.oneofs_by_name['action_oneof'].fields.append(
  _ACTION.fields_by_name['clickAndWaitForNavigationAction'])
_ACTION.fields_by_name['clickAndWaitForNavigationAction'].containing_oneof = _ACTION.oneofs_by_name['action_oneof']
_ACTION.oneofs_by_name['action_oneof'].fields.append(
  _ACTION.fields_by_name['closeAction'])
_ACTION.fields_by_name['closeAction'].containing_oneof = _ACTION.oneofs_by_name['action_oneof']
_ACTION.oneofs_by_name['action_oneof'].fields.append(
  _ACTION.fields_by_name['cookiesAction'])
_ACTION.fields_by_name['cookiesAction'].containing_oneof = _ACTION.oneofs_by_name['action_oneof']
_ACTION.oneofs_by_name['action_oneof'].fields.append(
  _ACTION.fields_by_name['deleteCookieAction'])
_ACTION.fields_by_name['deleteCookieAction'].containing_oneof = _ACTION.oneofs_by_name['action_oneof']
_ACTION.oneofs_by_name['action_oneof'].fields.append(
  _ACTION.fields_by_name['emulateAction'])
_ACTION.fields_by_name['emulateAction'].containing_oneof = _ACTION.oneofs_by_name['action_oneof']
_ACTION.oneofs_by_name['action_oneof'].fields.append(
  _ACTION.fields_by_name['emulateMediaTypeAction'])
_ACTION.fields_by_name['emulateMediaTypeAction'].containing_oneof = _ACTION.oneofs_by_name['action_oneof']
_ACTION.oneofs_by_name['action_oneof'].fields.append(
  _ACTION.fields_by_name['evaluateAction'])
_ACTION.fields_by_name['evaluateAction'].containing_oneof = _ACTION.oneofs_by_name['action_oneof']
_ACTION.oneofs_by_name['action_oneof'].fields.append(
  _ACTION.fields_by_name['focusAction'])
_ACTION.fields_by_name['focusAction'].containing_oneof = _ACTION.oneofs_by_name['action_oneof']
_ACTION.oneofs_by_name['action_oneof'].fields.append(
  _ACTION.fields_by_name['getInnerTextAction'])
_ACTION.fields_by_name['getInnerTextAction'].containing_oneof = _ACTION.oneofs_by_name['action_oneof']
_ACTION.oneofs_by_name['action_oneof'].fields.append(
  _ACTION.fields_by_name['gotoAction'])
_ACTION.fields_by_name['gotoAction'].containing_oneof = _ACTION.oneofs_by_name['action_oneof']
_ACTION.oneofs_by_name['action_oneof'].fields.append(
  _ACTION.fields_by_name['keyDownAction'])
_ACTION.fields_by_name['keyDownAction'].containing_oneof = _ACTION.oneofs_by_name['action_oneof']
_ACTION.oneofs_by_name['action_oneof'].fields.append(
  _ACTION.fields_by_name['keyPressAction'])
_ACTION.fields_by_name['keyPressAction'].containing_oneof = _ACTION.oneofs_by_name['action_oneof']
_ACTION.oneofs_by_name['action_oneof'].fields.append(
  _ACTION.fields_by_name['keyUpAction'])
_ACTION.fields_by_name['keyUpAction'].containing_oneof = _ACTION.oneofs_by_name['action_oneof']
_ACTION.oneofs_by_name['action_oneof'].fields.append(
  _ACTION.fields_by_name['launchAction'])
_ACTION.fields_by_name['launchAction'].containing_oneof = _ACTION.oneofs_by_name['action_oneof']
_ACTION.oneofs_by_name['action_oneof'].fields.append(
  _ACTION.fields_by_name['pdfAction'])
_ACTION.fields_by_name['pdfAction'].containing_oneof = _ACTION.oneofs_by_name['action_oneof']
_ACTION.oneofs_by_name['action_oneof'].fields.append(
  _ACTION.fields_by_name['reloadAction'])
_ACTION.fields_by_name['reloadAction'].containing_oneof = _ACTION.oneofs_by_name['action_oneof']
_ACTION.oneofs_by_name['action_oneof'].fields.append(
  _ACTION.fields_by_name['selectAction'])
_ACTION.fields_by_name['selectAction'].containing_oneof = _ACTION.oneofs_by_name['action_oneof']
_ACTION.oneofs_by_name['action_oneof'].fields.append(
  _ACTION.fields_by_name['setCookieAction'])
_ACTION.fields_by_name['setCookieAction'].containing_oneof = _ACTION.oneofs_by_name['action_oneof']
_ACTION.oneofs_by_name['action_oneof'].fields.append(
  _ACTION.fields_by_name['setViewportAction'])
_ACTION.fields_by_name['setViewportAction'].containing_oneof = _ACTION.oneofs_by_name['action_oneof']
_ACTION.oneofs_by_name['action_oneof'].fields.append(
  _ACTION.fields_by_name['startTracingAction'])
_ACTION.fields_by_name['startTracingAction'].containing_oneof = _ACTION.oneofs_by_name['action_oneof']
_ACTION.oneofs_by_name['action_oneof'].fields.append(
  _ACTION.fields_by_name['stopTracingAction'])
_ACTION.fields_by_name['stopTracingAction'].containing_oneof = _ACTION.oneofs_by_name['action_oneof']
_ACTION.oneofs_by_name['action_oneof'].fields.append(
  _ACTION.fields_by_name['screenshotAction'])
_ACTION.fields_by_name['screenshotAction'].containing_oneof = _ACTION.oneofs_by_name['action_oneof']
_ACTION.oneofs_by_name['action_oneof'].fields.append(
  _ACTION.fields_by_name['tapAction'])
_ACTION.fields_by_name['tapAction'].containing_oneof = _ACTION.oneofs_by_name['action_oneof']
_ACTION.oneofs_by_name['action_oneof'].fields.append(
  _ACTION.fields_by_name['typeAction'])
_ACTION.fields_by_name['typeAction'].containing_oneof = _ACTION.oneofs_by_name['action_oneof']
_ACTION.oneofs_by_name['action_oneof'].fields.append(
  _ACTION.fields_by_name['waitAction'])
_ACTION.fields_by_name['waitAction'].containing_oneof = _ACTION.oneofs_by_name['action_oneof']
_ACTION.oneofs_by_name['action_oneof'].fields.append(
  _ACTION.fields_by_name['waitForAction'])
_ACTION.fields_by_name['waitForAction'].containing_oneof = _ACTION.oneofs_by_name['action_oneof']
_ACTION.oneofs_by_name['action_oneof'].fields.append(
  _ACTION.fields_by_name['uploadFileAction'])
_ACTION.fields_by_name['uploadFileAction'].containing_oneof = _ACTION.oneofs_by_name['action_oneof']
_CLICKACTION.fields_by_name['option'].message_type = _CLICKOPTION
_CLICKANDWAITFORNAVIGATIONACTION.fields_by_name['option'].message_type = _CLICKOPTION
_CLICKANDWAITFORNAVIGATIONACTION.fields_by_name['waitOption'].message_type = _WAITOPTION
_DOREQUEST.fields_by_name['actions'].message_type = _ACTION
_DORESPONSE.fields_by_name['status'].enum_type = _DORESPONSE_STATUS
_DORESPONSE_STATUS.containing_type = _DORESPONSE
_LAUNCHACTION.fields_by_name['defaultViewport'].message_type = _VIEWPORT
DESCRIPTOR.message_types_by_name['Action'] = _ACTION
DESCRIPTOR.message_types_by_name['ClearAndTypeAction'] = _CLEARANDTYPEACTION
DESCRIPTOR.message_types_by_name['ClickAction'] = _CLICKACTION
DESCRIPTOR.message_types_by_name['ClickAndWaitForNavigationAction'] = _CLICKANDWAITFORNAVIGATIONACTION
DESCRIPTOR.message_types_by_name['CloseAction'] = _CLOSEACTION
DESCRIPTOR.message_types_by_name['CookiesAction'] = _COOKIESACTION
DESCRIPTOR.message_types_by_name['DeleteCookieAction'] = _DELETECOOKIEACTION
DESCRIPTOR.message_types_by_name['DoRequest'] = _DOREQUEST
DESCRIPTOR.message_types_by_name['DoResponse'] = _DORESPONSE
DESCRIPTOR.message_types_by_name['EmulateAction'] = _EMULATEACTION
DESCRIPTOR.message_types_by_name['EmulateMediaTypeAction'] = _EMULATEMEDIATYPEACTION
DESCRIPTOR.message_types_by_name['EvaluateAction'] = _EVALUATEACTION
DESCRIPTOR.message_types_by_name['FocusAction'] = _FOCUSACTION
DESCRIPTOR.message_types_by_name['GetInnerTextAction'] = _GETINNERTEXTACTION
DESCRIPTOR.message_types_by_name['GotoAction'] = _GOTOACTION
DESCRIPTOR.message_types_by_name['KeyDownAction'] = _KEYDOWNACTION
DESCRIPTOR.message_types_by_name['KeyPressAction'] = _KEYPRESSACTION
DESCRIPTOR.message_types_by_name['KeyUpAction'] = _KEYUPACTION
DESCRIPTOR.message_types_by_name['LaunchAction'] = _LAUNCHACTION
DESCRIPTOR.message_types_by_name['PdfAction'] = _PDFACTION
DESCRIPTOR.message_types_by_name['ReloadAction'] = _RELOADACTION
DESCRIPTOR.message_types_by_name['SelectAction'] = _SELECTACTION
DESCRIPTOR.message_types_by_name['SetCookieAction'] = _SETCOOKIEACTION
DESCRIPTOR.message_types_by_name['SetViewportAction'] = _SETVIEWPORTACTION
DESCRIPTOR.message_types_by_name['StartTracingAction'] = _STARTTRACINGACTION
DESCRIPTOR.message_types_by_name['StopTracingAction'] = _STOPTRACINGACTION
DESCRIPTOR.message_types_by_name['ScreenshotAction'] = _SCREENSHOTACTION
DESCRIPTOR.message_types_by_name['TapAction'] = _TAPACTION
DESCRIPTOR.message_types_by_name['TypeAction'] = _TYPEACTION
DESCRIPTOR.message_types_by_name['WaitAction'] = _WAITACTION
DESCRIPTOR.message_types_by_name['WaitForAction'] = _WAITFORACTION
DESCRIPTOR.message_types_by_name['UploadFileAction'] = _UPLOADFILEACTION
DESCRIPTOR.message_types_by_name['ClickOption'] = _CLICKOPTION
DESCRIPTOR.message_types_by_name['WaitOption'] = _WAITOPTION
DESCRIPTOR.message_types_by_name['Viewport'] = _VIEWPORT
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Action = _reflection.GeneratedProtocolMessageType('Action', (_message.Message,), {
  'DESCRIPTOR' : _ACTION,
  '__module__' : 'px.px_pb2'
  # @@protoc_insertion_point(class_scope:px.Action)
  })
_sym_db.RegisterMessage(Action)

ClearAndTypeAction = _reflection.GeneratedProtocolMessageType('ClearAndTypeAction', (_message.Message,), {
  'DESCRIPTOR' : _CLEARANDTYPEACTION,
  '__module__' : 'px.px_pb2'
  # @@protoc_insertion_point(class_scope:px.ClearAndTypeAction)
  })
_sym_db.RegisterMessage(ClearAndTypeAction)

ClickAction = _reflection.GeneratedProtocolMessageType('ClickAction', (_message.Message,), {
  'DESCRIPTOR' : _CLICKACTION,
  '__module__' : 'px.px_pb2'
  # @@protoc_insertion_point(class_scope:px.ClickAction)
  })
_sym_db.RegisterMessage(ClickAction)

ClickAndWaitForNavigationAction = _reflection.GeneratedProtocolMessageType('ClickAndWaitForNavigationAction', (_message.Message,), {
  'DESCRIPTOR' : _CLICKANDWAITFORNAVIGATIONACTION,
  '__module__' : 'px.px_pb2'
  # @@protoc_insertion_point(class_scope:px.ClickAndWaitForNavigationAction)
  })
_sym_db.RegisterMessage(ClickAndWaitForNavigationAction)

CloseAction = _reflection.GeneratedProtocolMessageType('CloseAction', (_message.Message,), {
  'DESCRIPTOR' : _CLOSEACTION,
  '__module__' : 'px.px_pb2'
  # @@protoc_insertion_point(class_scope:px.CloseAction)
  })
_sym_db.RegisterMessage(CloseAction)

CookiesAction = _reflection.GeneratedProtocolMessageType('CookiesAction', (_message.Message,), {
  'DESCRIPTOR' : _COOKIESACTION,
  '__module__' : 'px.px_pb2'
  # @@protoc_insertion_point(class_scope:px.CookiesAction)
  })
_sym_db.RegisterMessage(CookiesAction)

DeleteCookieAction = _reflection.GeneratedProtocolMessageType('DeleteCookieAction', (_message.Message,), {
  'DESCRIPTOR' : _DELETECOOKIEACTION,
  '__module__' : 'px.px_pb2'
  # @@protoc_insertion_point(class_scope:px.DeleteCookieAction)
  })
_sym_db.RegisterMessage(DeleteCookieAction)

DoRequest = _reflection.GeneratedProtocolMessageType('DoRequest', (_message.Message,), {
  'DESCRIPTOR' : _DOREQUEST,
  '__module__' : 'px.px_pb2'
  # @@protoc_insertion_point(class_scope:px.DoRequest)
  })
_sym_db.RegisterMessage(DoRequest)

DoResponse = _reflection.GeneratedProtocolMessageType('DoResponse', (_message.Message,), {
  'DESCRIPTOR' : _DORESPONSE,
  '__module__' : 'px.px_pb2'
  # @@protoc_insertion_point(class_scope:px.DoResponse)
  })
_sym_db.RegisterMessage(DoResponse)

EmulateAction = _reflection.GeneratedProtocolMessageType('EmulateAction', (_message.Message,), {
  'DESCRIPTOR' : _EMULATEACTION,
  '__module__' : 'px.px_pb2'
  # @@protoc_insertion_point(class_scope:px.EmulateAction)
  })
_sym_db.RegisterMessage(EmulateAction)

EmulateMediaTypeAction = _reflection.GeneratedProtocolMessageType('EmulateMediaTypeAction', (_message.Message,), {
  'DESCRIPTOR' : _EMULATEMEDIATYPEACTION,
  '__module__' : 'px.px_pb2'
  # @@protoc_insertion_point(class_scope:px.EmulateMediaTypeAction)
  })
_sym_db.RegisterMessage(EmulateMediaTypeAction)

EvaluateAction = _reflection.GeneratedProtocolMessageType('EvaluateAction', (_message.Message,), {
  'DESCRIPTOR' : _EVALUATEACTION,
  '__module__' : 'px.px_pb2'
  # @@protoc_insertion_point(class_scope:px.EvaluateAction)
  })
_sym_db.RegisterMessage(EvaluateAction)

FocusAction = _reflection.GeneratedProtocolMessageType('FocusAction', (_message.Message,), {
  'DESCRIPTOR' : _FOCUSACTION,
  '__module__' : 'px.px_pb2'
  # @@protoc_insertion_point(class_scope:px.FocusAction)
  })
_sym_db.RegisterMessage(FocusAction)

GetInnerTextAction = _reflection.GeneratedProtocolMessageType('GetInnerTextAction', (_message.Message,), {
  'DESCRIPTOR' : _GETINNERTEXTACTION,
  '__module__' : 'px.px_pb2'
  # @@protoc_insertion_point(class_scope:px.GetInnerTextAction)
  })
_sym_db.RegisterMessage(GetInnerTextAction)

GotoAction = _reflection.GeneratedProtocolMessageType('GotoAction', (_message.Message,), {
  'DESCRIPTOR' : _GOTOACTION,
  '__module__' : 'px.px_pb2'
  # @@protoc_insertion_point(class_scope:px.GotoAction)
  })
_sym_db.RegisterMessage(GotoAction)

KeyDownAction = _reflection.GeneratedProtocolMessageType('KeyDownAction', (_message.Message,), {
  'DESCRIPTOR' : _KEYDOWNACTION,
  '__module__' : 'px.px_pb2'
  # @@protoc_insertion_point(class_scope:px.KeyDownAction)
  })
_sym_db.RegisterMessage(KeyDownAction)

KeyPressAction = _reflection.GeneratedProtocolMessageType('KeyPressAction', (_message.Message,), {
  'DESCRIPTOR' : _KEYPRESSACTION,
  '__module__' : 'px.px_pb2'
  # @@protoc_insertion_point(class_scope:px.KeyPressAction)
  })
_sym_db.RegisterMessage(KeyPressAction)

KeyUpAction = _reflection.GeneratedProtocolMessageType('KeyUpAction', (_message.Message,), {
  'DESCRIPTOR' : _KEYUPACTION,
  '__module__' : 'px.px_pb2'
  # @@protoc_insertion_point(class_scope:px.KeyUpAction)
  })
_sym_db.RegisterMessage(KeyUpAction)

LaunchAction = _reflection.GeneratedProtocolMessageType('LaunchAction', (_message.Message,), {
  'DESCRIPTOR' : _LAUNCHACTION,
  '__module__' : 'px.px_pb2'
  # @@protoc_insertion_point(class_scope:px.LaunchAction)
  })
_sym_db.RegisterMessage(LaunchAction)

PdfAction = _reflection.GeneratedProtocolMessageType('PdfAction', (_message.Message,), {
  'DESCRIPTOR' : _PDFACTION,
  '__module__' : 'px.px_pb2'
  # @@protoc_insertion_point(class_scope:px.PdfAction)
  })
_sym_db.RegisterMessage(PdfAction)

ReloadAction = _reflection.GeneratedProtocolMessageType('ReloadAction', (_message.Message,), {
  'DESCRIPTOR' : _RELOADACTION,
  '__module__' : 'px.px_pb2'
  # @@protoc_insertion_point(class_scope:px.ReloadAction)
  })
_sym_db.RegisterMessage(ReloadAction)

SelectAction = _reflection.GeneratedProtocolMessageType('SelectAction', (_message.Message,), {
  'DESCRIPTOR' : _SELECTACTION,
  '__module__' : 'px.px_pb2'
  # @@protoc_insertion_point(class_scope:px.SelectAction)
  })
_sym_db.RegisterMessage(SelectAction)

SetCookieAction = _reflection.GeneratedProtocolMessageType('SetCookieAction', (_message.Message,), {
  'DESCRIPTOR' : _SETCOOKIEACTION,
  '__module__' : 'px.px_pb2'
  # @@protoc_insertion_point(class_scope:px.SetCookieAction)
  })
_sym_db.RegisterMessage(SetCookieAction)

SetViewportAction = _reflection.GeneratedProtocolMessageType('SetViewportAction', (_message.Message,), {
  'DESCRIPTOR' : _SETVIEWPORTACTION,
  '__module__' : 'px.px_pb2'
  # @@protoc_insertion_point(class_scope:px.SetViewportAction)
  })
_sym_db.RegisterMessage(SetViewportAction)

StartTracingAction = _reflection.GeneratedProtocolMessageType('StartTracingAction', (_message.Message,), {
  'DESCRIPTOR' : _STARTTRACINGACTION,
  '__module__' : 'px.px_pb2'
  # @@protoc_insertion_point(class_scope:px.StartTracingAction)
  })
_sym_db.RegisterMessage(StartTracingAction)

StopTracingAction = _reflection.GeneratedProtocolMessageType('StopTracingAction', (_message.Message,), {
  'DESCRIPTOR' : _STOPTRACINGACTION,
  '__module__' : 'px.px_pb2'
  # @@protoc_insertion_point(class_scope:px.StopTracingAction)
  })
_sym_db.RegisterMessage(StopTracingAction)

ScreenshotAction = _reflection.GeneratedProtocolMessageType('ScreenshotAction', (_message.Message,), {
  'DESCRIPTOR' : _SCREENSHOTACTION,
  '__module__' : 'px.px_pb2'
  # @@protoc_insertion_point(class_scope:px.ScreenshotAction)
  })
_sym_db.RegisterMessage(ScreenshotAction)

TapAction = _reflection.GeneratedProtocolMessageType('TapAction', (_message.Message,), {
  'DESCRIPTOR' : _TAPACTION,
  '__module__' : 'px.px_pb2'
  # @@protoc_insertion_point(class_scope:px.TapAction)
  })
_sym_db.RegisterMessage(TapAction)

TypeAction = _reflection.GeneratedProtocolMessageType('TypeAction', (_message.Message,), {
  'DESCRIPTOR' : _TYPEACTION,
  '__module__' : 'px.px_pb2'
  # @@protoc_insertion_point(class_scope:px.TypeAction)
  })
_sym_db.RegisterMessage(TypeAction)

WaitAction = _reflection.GeneratedProtocolMessageType('WaitAction', (_message.Message,), {
  'DESCRIPTOR' : _WAITACTION,
  '__module__' : 'px.px_pb2'
  # @@protoc_insertion_point(class_scope:px.WaitAction)
  })
_sym_db.RegisterMessage(WaitAction)

WaitForAction = _reflection.GeneratedProtocolMessageType('WaitForAction', (_message.Message,), {
  'DESCRIPTOR' : _WAITFORACTION,
  '__module__' : 'px.px_pb2'
  # @@protoc_insertion_point(class_scope:px.WaitForAction)
  })
_sym_db.RegisterMessage(WaitForAction)

UploadFileAction = _reflection.GeneratedProtocolMessageType('UploadFileAction', (_message.Message,), {
  'DESCRIPTOR' : _UPLOADFILEACTION,
  '__module__' : 'px.px_pb2'
  # @@protoc_insertion_point(class_scope:px.UploadFileAction)
  })
_sym_db.RegisterMessage(UploadFileAction)

ClickOption = _reflection.GeneratedProtocolMessageType('ClickOption', (_message.Message,), {
  'DESCRIPTOR' : _CLICKOPTION,
  '__module__' : 'px.px_pb2'
  # @@protoc_insertion_point(class_scope:px.ClickOption)
  })
_sym_db.RegisterMessage(ClickOption)

WaitOption = _reflection.GeneratedProtocolMessageType('WaitOption', (_message.Message,), {
  'DESCRIPTOR' : _WAITOPTION,
  '__module__' : 'px.px_pb2'
  # @@protoc_insertion_point(class_scope:px.WaitOption)
  })
_sym_db.RegisterMessage(WaitOption)

Viewport = _reflection.GeneratedProtocolMessageType('Viewport', (_message.Message,), {
  'DESCRIPTOR' : _VIEWPORT,
  '__module__' : 'px.px_pb2'
  # @@protoc_insertion_point(class_scope:px.Viewport)
  })
_sym_db.RegisterMessage(Viewport)


DESCRIPTOR._options = None

_BROWSER = _descriptor.ServiceDescriptor(
  name='Browser',
  full_name='px.Browser',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=3015,
  serialized_end=3063,
  methods=[
  _descriptor.MethodDescriptor(
    name='Do',
    full_name='px.Browser.Do',
    index=0,
    containing_service=None,
    input_type=_DOREQUEST,
    output_type=_DORESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_BROWSER)

DESCRIPTOR.services_by_name['Browser'] = _BROWSER

# @@protoc_insertion_point(module_scope)



import urllib.request
import zipfile
import os, stat, errno, subprocess, socket, platform
import logging
from pathlib import Path

logging.getLogger().setLevel(logging.INFO)

px_server_package_download_root_url = 'https://ckpx.s3.amazonaws.com/px-server'

windows_px_server_package_download_url_template = px_server_package_download_root_url + '/{version}/px-windows.zip'

mac_px_server_package_download_url_template = px_server_package_download_root_url + '/{version}/px-mac.zip'

linux_px_server_package_download_url_template = px_server_package_download_root_url + '/{version}/px-linux.zip'

px_home_dir = os.path.join(Path.home(), '.px')

default_px_server_package_path = os.path.join(px_home_dir, 'px-server')

default_os_name = platform.system()

default_px_server_version = '0.0.1'

def download_and_extract_px_server_package(os_name=None, version=None, px_server_package_path=None):
    if os_name is None:
        os_name = default_os_name

    if version is None:
        version = default_px_server_version

    px_server_package_download_url = get_px_server_package_download_url(os_name=os_name, version=version)

    if px_server_package_path is None:
        px_server_package_path = default_px_server_package_path

    px_server_package_path = os.path.join(px_server_package_path, '{os}-{version}'.format(os=os_name, version=version))

    if os_name == 'Windows':
        px_server_executable_file_name = 'px.exe'
    else:
        px_server_executable_file_name = 'px'
    px_server_executable_file_path = os.path.join(px_server_package_path, px_server_executable_file_name)
    grpc_file_path = os.path.join(px_server_package_path, 'grpc_node.node')

    if not os.path.exists(px_server_executable_file_path) or not os.path.exists(grpc_file_path):
        try:
            os.remove(grpc_file_path)
        except OSError as e:
            if e.errno != errno.ENOENT:
                raise

        try:
            os.remove(px_server_executable_file_path)
        except OSError as e:
            if e.errno != errno.ENOENT:
                raise

        logging.info('Download px server package from {}'.format(px_server_package_download_url))
        file_name, header = urllib.request.urlretrieve(px_server_package_download_url)

        logging.info('Extracting px server package from {} to {}'.format(file_name, px_server_package_path))
        with zipfile.ZipFile(file_name, 'r') as zip_ref:
            zip_ref.extractall(px_server_package_path)

        try:
            logging.info('Delete px server package {}'.format(file_name))
            os.remove(file_name)
        except OSError:
            pass

    os.chmod(grpc_file_path, 0o755)
    os.chmod(px_server_executable_file_path, 0o755)

    return px_server_executable_file_path

def get_px_server_package_download_url(os_name=None, version=None):
    if os_name is None:
        os_name = default_os_name

    if version is None:
        version = default_px_server_version

    if os_name == 'Windows':
        px_server_package_download_url_template = windows_px_server_package_download_url_template
    elif os_name == 'Linux':
        px_server_package_download_url_template = linux_px_server_package_download_url_template
    elif os_name == 'Darwin':
        px_server_package_download_url_template = mac_px_server_package_download_url_template
    else:
        raise Exception('Unsupported OS: {}'.format(os_name))

    return px_server_package_download_url_template.format(version=version)



import grpc
import px.px_pb2 as px
from px.px_pb2_grpc import BrowserStub
from px_server.server import Server
import logging

logging.getLogger().setLevel(logging.INFO)


class Client:

    channel = None

    stub = None

    debug = False

    server = None

    server_url = None

    def __init__(self, server_url=None, debug=False, new_server=False, server_port=None):
        if server_url is None:
            if new_server is True:
                self.server = Server(server_port)
            else:
                self.server = Server.get_instance(server_port)
            server_url = 'localhost:{}'.format(self.server.port)

        self.server_url = server_url
        self.channel = grpc.insecure_channel(self.server_url)
        grpc.channel_ready_future(self.channel).result(timeout=300)
        self.stub = BrowserStub(self.channel)
        self.debug = debug

    def __del__(self):
        if self.channel is not None:
            self.channel.close()
            logging.info('Channel closed')

    def do_request(self, actions):
        request = px.DoRequest(actions=[actions])
        response = self.stub.Do(request)
        if response.status == 1:
            raise Exception(response.error)
        else:
            return response.result

    def clear_and_type(self, selector, text):
        actions = px.Action(clearAndTypeAction=px.ClearAndTypeAction(selector=selector, text=text))
        self.do_request(actions)

    def click(self, selector, option=None):
        actions = px.Action(clickAction=px.ClickAction(selector=selector, option=option))
        self.do_request(actions)

    def clickAndWaitForNavigation(self, selector, option=None, waitOption=None):
        actions = px.Action(clickAndWaitForNavigationAction=px.ClickAndWaitForNavigationAction(selector=selector, option=option, waitOption=waitOption))
        self.do_request(actions)

    def close(self):
        actions = px.Action(closeAction=px.CloseAction())
        self.do_request(actions)

    def cookies(self):
        actions = px.Action(cookiesAction=px.CookiesAction())
        self.do_request(actions)

    def delete_cookies(self):
        actions = px.Action(deleteCookiesAction=px.DeleteCookiesAction())
        self.do_request(actions)

    def get_text(self, selector):
        actions = px.Action(getInnerTextAction=px.GetInnerTextAction(selector=selector))
        return self.do_request(actions)

    def goto(self, url):
        actions = px.Action(gotoAction=px.GotoAction(url=url))
        self.do_request(actions)

    def keyDown(self, key):
        actions = px.Action(keyDownAction=px.KeyDownAction(key=key))
        self.do_request(actions)

    def keyPress(self, key):
        actions = px.Action(keyPressAction=px.KeyPressAction(key=key))
        self.do_request(actions)

    def keyUp(self, key):
        actions = px.Action(keyUpAction=px.KeyUpAction(key=key))
        self.do_request(actions)

    def launch(self, headless=None, defaultViewport=None, args=[]):
        actions = px.Action(launchAction=px.LaunchAction(headless=headless, defaultViewport=defaultViewport, args=args))
        self.do_request(actions)

    def select(self, selector, values):
        actions = px.Action(selectAction=px.SelectAction(selector=selector, values=values))
        self.do_request(actions)

    def screenshot(self, path, fullPage):
        actions = px.Action(screenshotAction=px.ScreenshotAction(path=path, fullPage=fullPage))
        self.do_request(actions)

    def reload(self):
        actions = px.Action(reloadAction=px.ReloadAction())
        self.do_request(actions)

    def type(self, selector, text):
        actions = px.Action(typeAction=px.TypeAction(selector=selector, text=text))
        self.do_request(actions)

    def wait(self, time):
        actions = px.Action(waitAction=px.WaitAction(time=time))
        self.do_request(actions)

    def waitFor(self, selector):
        actions = px.Action(waitForAction=px.WaitForAction(selector=selector))
        return self.do_request(actions)

    def uploadFile(self, selector, paths):
        actions = px.Action(uploadFileAction=px.UploadFileAction(selector=selector, paths=paths))
        self.do_request(actions)



from setuptools import setup, find_packages
from setuptools.command.install import install
from px_server.helper import download_and_extract_px_server_package
from datetime import datetime

required_pkgs = ['grpcio>=1.27.2', 'grpcio-tools>=1.27.2']

class CustomInstall(install):
    def run(self):
        install.run(self)
        download_and_extract_px_server_package()


setup(
    name='ckpx',
    version='0.0.' + datetime.today().strftime('%y%d%m.%H%M'),
    packages=find_packages(),
    setup_requires=required_pkgs,
    install_requires=required_pkgs,
    author='p2trx',
    description='px',
    url='https://github.com/p2trx/px',
    # include_package_data=True,
    # package_data={
    #     'package': ['px_server/package/**/*'],
    # },
    cmdclass={'install': CustomInstall},
)
